#!/usr/bin/env perl
#
# This script is a part of Genome Search Toolkit
# Copyright (c) 2015 Kirill Kryukov
# see LICENSE.txt for details.
#

#
# Input: "name.best-hit-taxid-n-queries".
# Output: "name.best-hit-taxid-n-queries-at-all-nodes".
#

use strict;
use warnings;
use Getopt::Long qw(GetOptionsFromArray :config pass_through);
use FindBin;
use lib "$FindBin::Bin";
use gst_utils;

gst_init();
if (@opts) { die "Unrecognized option \"$opts[0]\"\n"; }
$help_str .= qq{Options:
    --help               - Show help
    --version            - Show version
};
print_help();

load_genome_taxonomy();

my (%unknown_nodes,%n_queries_per_taxid,%n_queries_per_taxid_combined);
my ($n_queries_total,$n_queries_have_hits,$n_queries_have_no_hits,$n_fractional) = (0,0,0,0);

binmode(STDIN);
while (<STDIN>)
{
    s/[\x0D\x0A]+$//;
    if (substr($_,0,1) eq '#') { next; }
    if (!/^(\d+|\d+\.\d+)\s+(\d+)\s(\S.*)$/) { print STDERR "Parse error: Can't parse \"$_\"\n"; next; }
    my ($n,$taxid,$taxon) = ($1,$2,$3);
    if ($taxid)
    {
        if (!exists $nodes{$taxid}) { $unknown_nodes{$taxid} = $taxon; next; }
        $n_queries_have_hits += $n;
        $n_queries_per_taxid{$taxid} += $n;
        if ($n =~ /\./) { $n_fractional++; }
    }
    else { $n_queries_have_no_hits = $n; }
    $n_queries_total += $n;
}

foreach my $leaf (keys %n_queries_per_taxid)
{
    my $n = $n_queries_per_taxid{$leaf};
    for (my $t = $leaf; $t; $t = $nodes{$t}->{'p'}) { $n_queries_per_taxid_combined{$t} += $n; }
}

if (scalar keys %unknown_nodes) { print STDERR scalar(keys %unknown_nodes), " unknown nodes found\n"; }

binmode(STDOUT);
print "# Number of taxon-pecific queries by taxon\n";
print "# Columns: number of queries, taxonomic id, taxonomic name\n";
print "$n_queries_have_no_hits 0 -\n";
my $format = $n_fractional ? '%.6f' : '%.0f';
foreach my $taxid (sort { $n_queries_per_taxid_combined{$b} <=> $n_queries_per_taxid_combined{$a} } keys %n_queries_per_taxid_combined)
{
    print sprintf($format,$n_queries_per_taxid_combined{$taxid}), ' ', $taxid, ' ', $nodes{$taxid}->{'n'}, "\n";
}
