#!/usr/bin/env perl

use strict;
use warnings;
use Getopt::Long;
use Storable;

my ($start_time,$script_name,$script_version,$script_date,$script_years) = (time,'classifier','0.1.0','2016-07-26','2016');
$| = 1;

my ($summary_dir,$dot_n,$help,$ver);
GetOptions(
    'genome-summary-dir=s' => \$summary_dir,
    'dot=i'                => \$dot_n,
    'help'                 => \$help,
    'version'              => \$ver)
or die "Can't parse command line arguments\n";

my $ver_str = "$script_name, version $script_version, $script_date\nCopyright (c) $script_years Kirill Kryukov\n";
my $help_str = qq{Usage: $script_name <option>...
Options:
    --genome-summary-dir <DIR> - Specify genome summary directory
    --dot N                    - Print a dot every N queries
    --help                     - Show help
    --version                  - Show version
};

print (($ver ? $ver_str : ''), ($help ? $help_str : ''));
if (!defined($summary_dir)) { die "Genome summary directory is not specified (--genome-summary-dir option)\n"; }



# Loading taxonomy.
my %nodes              = %{retrieve(file_exists("$summary_dir/nodes-with-genome.hash"        ))};
my %name_to_taxid      = %{retrieve(file_exists("$summary_dir/genome-name-to-taxid.hash"     ))};
my %safe_name_to_taxid = %{retrieve(file_exists("$summary_dir/genome-safe-name-to-taxid.hash"))};

# Separate hash for storing parent node ids.
my %np;
foreach my $t (keys %nodes) { $np{$t} = $nodes{$t}->{'p'}; }

# Constructing hash for looking up taxonomic ID by name.
my %taxid_to_name;
foreach my $n (keys %name_to_taxid) { $taxid_to_name{$name_to_taxid{$n}} = $n; }
foreach my $n (keys %safe_name_to_taxid) { $taxid_to_name{$safe_name_to_taxid{$n}} = $n; }
my %all_name_to_taxid;
foreach my $t (keys %taxid_to_name) { $all_name_to_taxid{$taxid_to_name{$t}} = $t; }

# Indexed hash of names for faster lookup.
my @indexed_name_to_taxid;
foreach my $n (keys %all_name_to_taxid) { $indexed_name_to_taxid[ord(substr($n,0,1))]->{substr($n,1)} = $all_name_to_taxid{$n}; }



my ($qi,$qname,$sname,%hit_scores) = (-1,'','');
my $dot_next = $dot_n;
binmode STDIN;
while (<STDIN>)
{
    s/[\x0D\x0A]+$//;

    if (substr($_,0,1) eq ' ')
    {
        process_one_query();
        $qname = substr($_,1);
        $qi++;

        if (defined($dot_n) and $qi >= $dot_next) { print STDERR '.'; $dot_next += $dot_n; }

        %hit_scores = ();
        next;
    }

    if ($qi < 0) { next; }
    my $si = index($_,' ',0);
    my $score = substr($_,0,$si);
    my $ni = $si+1;
    my $sname = substr($_,$ni,rindex($_,' ')-$ni);
    my $sid = $indexed_name_to_taxid[ord(substr($sname,0,1))]->{substr($sname,1)};

    if (!$sid) { die "Unknown database name \"$sname\"\n"; }
    $hit_scores{$sid} = $score;
}
process_one_query();



sub process_one_query
{
    if ($qi < 0 or !scalar(keys %hit_scores)) { return; }

    print "\n$qname\n";
    foreach my $sid (keys %hit_scores)
    {
        print " - $sid - ", $taxid_to_name{$sid}, "\n";
    }
}

sub file_exists { my ($f) = @_; if (-e $f and -f $f) { return $f; } else { die "Error: Can't find file \"$f\""; } }
