#!/usr/bin/env perl
#
# This script is a part of Genome Search Toolkit
# Copyright (c) 2015 Kirill Kryukov
# see LICENSE.txt for details.
#

use strict;
use warnings;
use Getopt::Long;

my ($script_name,$script_version,$script_date,$script_years) = ('fasta-rename-to-numbers','0.1.1','2015-05-15','2015');

my ($first,$prefix,$base62,$help,$ver) = (1,'');
GetOptions(
    'first=i'  => \$first,
    'prefix=s' => \$prefix,
    'base62'   => \$base62,
    'help'     => \$help,
    'version'  => \$ver)
or die "Can't parse command line arguments\n";

my $ver_str = "$script_name, version $script_version, $script_date\nCopyright (c) $script_years Kirill Kryukov\n";
my $help_str = qq{Processes FASTA format, renaming all sequences into '1', '2', etc. Optional
'--prefix' allows to begin the name with any prefix string. E.g., with
'--prefix seq' the names will be 'seq1', 'seq2', etc.

Usage: $script_name <option>...
Options:
    --first <N>  - Start from number <N> (default: 1)
    --prefix <S> - Begin all names with <S> (default: no prefix)
    --help       - Show help
    --version    - Show version
};

if ($ver or $help) { print (($ver ? $ver_str : ''), ($help ? $help_str : '')); exit; }

my @base64_digits = ('0','1','2','3','4','5','6','7','8','9',
    'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
    'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');

binmode STDIN;
binmode STDOUT;

my $i = $first;
while (<STDIN>)
{
    s/[\x0D\x0A]+$//;
    if (substr($_,0,1) eq '>')
    {
        print '>', $prefix, ($base62 ? encode_base62($i) : $i), "\n";
        $i++;
    }
    else { print $_, "\n"; }
}

sub encode_base62
{
    my ($n) = @_;
    my $s = '';
    while ($n > 0 or $s eq '')
    {
        my $b = $n % 62;
        $n = int($n / 62);
        $s = $base64_digits[$b] . $s;
    }
    return $s;
}
