#!/usr/bin/env perl
#
# This script is a part of Genome Search Toolkit
# Copyright (c) 2016 Kirill Kryukov
# see LICENSE.txt for details.
#

use strict;
use warnings;
use Getopt::Long qw(GetOptionsFromArray :config pass_through);
use FindBin;
use lib "$FindBin::Bin";
use gst_utils;

gst_init();

my ($strict) = (0);
GetOptionsFromArray(\@opts,
    'strict'    => \$strict
);
if (@opts) { die "Unrecognized option \"$opts[0]\"\n"; }

$help_str .= qq{Options:
    --use-db-taxa <LIST> - Genome taxa to include (;;-separated)
    --strict             - Strict mode - stop on any unrecognized genome
    --help               - Show help
    --version            - Show version
};
print_help();

read_genome_info();
if (defined($use_db_taxa)) { load_genome_taxonomy(); }
mark_genomes_to_use_by_taxa();

binmode STDIN;
binmode STDOUT;

while (<STDIN>)
{
    s/[\x0D\x0A]+$//;
    my $name = $_;
    if (exists $genomes{$name})
    {
        if ($genomes{$name}->{'use'}) { print "$name\n"; }
    }
    else
    {
        if ($name =~ /^(.+?)\s+\[([^\[\]]+)\]$/)
        {
            my $safe_name = change_name_to_safe_name($1) . ' [' . change_name_to_safe_name($2) . ']';
            if (exists $genomes{$safe_name})
            {
                if ($genomes{$safe_name}->{'use'}) { print "$name\n"; }
            }
            else { if ($strict) { die "Unknown genome \"$name\"\n"; } }
        }
        else { if ($strict) { die "Can't parse genome name \"$name\"\n"; } }
    }
}
