#!/usr/bin/env perl
#
# This script is a part of Genome Search Toolkit
# Copyright (c) 2016 Kirill Kryukov
# see LICENSE.txt for details.
#

use strict;
use warnings;
use Getopt::Long qw(GetOptionsFromArray :config pass_through);
use FindBin;
use lib "$FindBin::Bin";
use gst_utils;

gst_init();

my ($list_file) = ();
GetOptionsFromArray(\@opts,
    'list-file=s' => \$list_file
);
if (@opts) { die "Unrecognized option \"$opts[0]\"\n"; }

$help_str .= qq{Options:
    --list-file  - List of genomes to skip
    --help       - Show help
    --version    - Show version
};
if (!defined $list_file) { $ver = 1; $help = 1; }
print_help();

if (!-e $list_file) { die "Can't find \"$list_file\"\n"; }

my %listed;
open(my $L,'<',$list_file) or die "Can't open \"$list_file\"\n";
binmode $L;
while (<$L>)
{
    s/[\x0D\x0A]+$//;
    if ($_ ne '') { $listed{$_} = 1; }
}
close $L;

binmode STDIN;
binmode STDOUT;

while (<STDIN>)
{
    s/[\x0D\x0A]+$//;
    if ($_ eq '') { next; }
    if (!exists $listed{$_}) { print $_, "\n"; }
}
