#!/usr/bin/env perl
#
# This script is a part of Genome Search Toolkit
# Copyright (c) 2014 Kirill Kryukov
# see LICENSE.txt for details.
#
# This script reads a non-cumulative taxonomic classification file.
# (Format: Number of reads, taxid)
#

use strict;
use warnings;
use FindBin;
use lib "$FindBin::Bin";
use gst_utils;

gst_init();
if (@opts) { die "Unrecognized option \"$opts[0]\"\n"; }

if (!defined($summary_dir)) { die "Genome summary directory is not specified (--genome-summary-dir option)\n"; }

load_genome_taxonomy();

binmode STDIN;
binmode STDOUT;

while (<STDIN>)
{
    if (substr($_,0,1) eq '#') { next; }
    my $i = index($_,' ');
    if ($i < 0) { die "Can't parse \"$_\"\n"; }
    my $nreads = substr($_,0,$i);
    my $rest = substr($_,$i+1);
    if ($rest !~ /^(\d+)/) { die "Can't parse \"$_\"\n"; }
    my $taxid = $1;
    #print "$nreads\t$taxid\n";

    if (!exists($nodes{$taxid})) { die "Unknown taxonomic id: $taxid\n"; }
    my @lineage;
    for (my $t = $taxid; $t != 1; $t = $nodes{$t}->{'p'})
    {
        push @lineage, $t;
    }

    print $nreads;
    foreach my $t (reverse @lineage)
    {
        print "\t", $nodes{$t}->{'n'};
    }
    print "\n";
}
