#!/usr/bin/env perl
#
# This script is a part of Genome Search Toolkit
# Copyright (c) 2014 Kirill Kryukov
# see LICENSE.txt for details.
#
# This script should be replaced by a combination of individual scripts doing separate tasks.
#
#     list-genome-names.pl
#     filter-genome-list-by-taxonomy.pl
#     partition-genome-list.pl
#

use strict;
use warnings;
use File::Basename qw(basename fileparse);
use File::Find;
use Getopt::Long qw(GetOptionsFromArray :config pass_through);
use FindBin;
use lib "$FindBin::Bin";
use gst_utils;

gst_init();

my ($dir,$ext);
GetOptionsFromArray(\@opts,
    'dir=s' => \$dir,
    'ext=s' => \$ext
);
if (@opts) { die "Unrecognized option \"$opts[0]\"\n"; }

$help_str .= qq{Options:
    --dir <DIR> - Directory to scan
    --ext <EXT> - Look for files with this extension (suffix)
    --help      - Show help
    --version   - Show version
};
if (!defined($dir)) { $ver = 1; $help = 1; }
print_help();

if (!-e $dir) { die "Can't find directory \"$dir\"\n"; }
if (!-d $dir) { die "\"$dir\" is not a directory\n"; }

read_genome_info();
if (defined($use_db_taxa)) { load_genome_taxonomy(); }
mark_genomes_to_use_by_taxa();

my %genome_list;

find({'wanted'=>\&{sub{ add_genome($_); }},'no_chdir'=>1},$dir);

sub add_genome
{
    my ($path) = @_;
    my $name = basename($path);
    if (defined($ext))
    {
        if ($name =~ /^(.+?)$ext$/) { $name = $1; }
        else { return; }
    }
    $name =~ s/\.$//;
    if (!exists $genomes{$name}) { return; }
    $genome_list{$name} = 1;
}

binmode STDOUT;

foreach my $gname (sort { unescape_file_name($a) cmp unescape_file_name($b) } keys %genome_list)
{
    print unescape_file_name($gname), "\n";
}
