#!C:\utl\dev\Strawberry-Perl-5.18.2.2-64-bit\perl\bin\perl.exe
#
# This script is a part of Genome Search Toolkit
# Copyright (c) 2014-2015 Kirill Kryukov
# see LICENSE.txt for details.
#

use strict;
use CGI qw/:standard/;
use File::Basename qw(basename dirname);
use File::Glob ':glob';
use HTML::Entities;
use Sort::Versions;
use Storable;
use gst_html;

# Configuration.
my $default_data_dir = 'R:\www\data\genome-search-toolkit\results';
my $default_data_dir_in_url = 'data/genome-search-toolkit/results';
my $summary_dir = 'N:\summary';
my @search_colors = ('','#2A55FF','#DD0909','#12BC12','#A323CD','#FFAA00');

# Initialize.
$| = 1;
binmode STDOUT;
$project_title = 'Best Hits Tree';
$page_title = 'Best Hits Tree';
html_load_fragments();
my %tax_rank_order = ();
read_tax_rank_order();

print header(-charset=>'utf-8'), process_template('{HTML header}');

my %nodes;

# Parameters.
my %searches = ();
my $dir = $default_data_dir;
my $dir_in_url = $default_data_dir_in_url;
my $lowest_rank_to_show = '';
my $min_n_queries_to_show_a_node = 0;
my $show_unranked_nodes = 0;
my $show_ranks = 0;
my $combine_identical_nodes = 0;
my $show_abs_number = 1;
my $show_percentage = 0;
my $show_percentage_out_of_all_queries = 0;
my $param_name = '';
my $param_value;
my %is_param_value;

if (param('dir'))
{
    $dir = $default_data_dir . '\\' . param('dir');
    $dir_in_url = $default_data_dir_in_url . '/' . param('dir');
}
if (!-e $dir or !-e $dir) { print "<div style=\"text-align: center\"><div class=\"error\">Can't find data directory</div></div>\n"; exit(0); }

# Locating all searches in this web-site.
my %all_searches = ();
foreach my $file (bsd_glob("$dir/*"))
{
    if (basename($file) !~ /^(.+?)\.([^\.]+)$/) { next; }
    my ($name,$ext) = ($1,$2);
    if ($ext eq 'best-hit-taxid-n-queries') { $all_searches{$name}->{'has-n-queries'} = 1; }
}

# Reading parameters.
my $ok = 1;
{ my $i = 1; for (param('s')) { if ($_ eq '') { next; } $searches{$_}->{'index'} = $i; $i++ } }
my $n_searches = scalar keys %searches;
if (!$n_searches) { print "<div style=\"text-align: center\"><div class=\"error\">No searches were selected</div></div>\n"; exit(0); }
for (keys %searches) { if (!exists $all_searches{$_} or !exists $all_searches{$_}->{'has-n-queries'}) { print 'Unknown search: "', encode_entities($_), "\"\n"; $ok = 0; } }
if (param('lowest_rank')) { ($lowest_rank_to_show) = param('lowest_rank'); if (!exists $tax_rank_order{$lowest_rank_to_show}) { $lowest_rank_to_show = ''; } }
if (param('min_queries')) { my ($n) = param('min_queries'); $min_n_queries_to_show_a_node = int($n); if ($min_n_queries_to_show_a_node < 0) { $min_n_queries_to_show_a_node = 0; } }
if (param('show_unranked_nodes')) { $show_unranked_nodes = 1; }
if (param('show_ranks')) { $show_ranks = 1; }
if (param('combine_identical_nodes')) { $combine_identical_nodes = 1; }
if (param('show_abs')) { $show_abs_number = 1; } else { $show_abs_number = 0; }
if (param('show_perc')) { $show_percentage = 1; } else { $show_percentage = 0; }
if (param('prec_rel')) { my ($a) = param('prec_rel'); $show_percentage_out_of_all_queries = ($a eq 'have_hits') ? 0 : 1; }
if (param('param')) { $param_value = param('param'); }

# Loading the taxonomy database.
my $nodes_file = "$dir/nodes-with-genome.hash";
if (!-e $nodes_file) { $nodes_file = "$summary_dir/nodes-with-genome.hash"; }
if ($ok and !-e $nodes_file) { print "Can't find taxonomy nodes file<br />\n"; $ok = 0; }
if ($ok) { %nodes = %{retrieve($nodes_file)}; if (!scalar keys %nodes) { print "Can't load taxonomy nodes<br />\n"; $ok = 0; } }
if ($ok) { compute_tax_order(1); }

my %unknown_nodes = ();

my %krona_urls;

# Loading the best hits data.
if ($ok)
{
A:  foreach my $search_name (keys %searches)
    {
        #print "<b>$search_name</b><br />\n";
        $searches{$search_name}->{'n_queries_have_hits'} = 0;
        %{$searches{$search_name}->{query_count_by_best_hit_taxid}} = ();
        my $best_hits_file = "$dir/$search_name.best-hit-taxid-n-queries";
        my $cumulate = 1;
        my ($nreads_column_index,$taxid_column_index) = (0,1);
        my $IN;
        if (!open($IN,'<',$best_hits_file)) { print "Can't load data for \"", encode_entities($search_name), "\"\n"; $ok = 0; last; }
        binmode $IN;
        while (<$IN>)
        {
            s/[\x0D\x0A]+$//;

            if (substr($_,0,1) eq '#')
            {
                if (/^#\s*cumulative/i) { $cumulate = 0; }
                elsif (/^#\s*non-cumulative/i) { $cumulate = 1; }
                elsif (/^#\s*parameter\sname:\s*(.+)/i) { if (!defined($param_name) or $param_name eq '') { $param_name = $1; } }
                elsif (/^#\s*(\d+)/)
                {
                    if (!defined($param_value)) { $param_value = $1; $nreads_column_index = 0; }
                    my $map = $_;
                    $map =~ s/^#\s*//;
                    my @fields = split("\t",$map);
                    foreach my $i (keys @fields)
                    {
                        if (defined($param_value) and $param_value eq $fields[$i]) { $nreads_column_index = $i; }
                        if ($fields[$i] =~ /^taxid$/i) { $taxid_column_index = $i; }
                        elsif ($fields[$i] =~ /^taxname$/i) {}
                        elsif ($fields[$i] =~ /\d+/) { $is_param_value{$fields[$i]} = 1; }
                    }
                }
                next;
            }

            my ($n,$taxid,$taxon);
            if ($taxid_column_index == 1)
            {
                if (!/^(\d+|\d+\.\d+)\s+(\d+)\s(\S.*)$/) { print "Parse error: Can't parse \"", encode_entities($_), "\"<br />\n"; next; }
                ($n,$taxid,$taxon) = ($1,$2,$3);
            }
            else
            {
                my @fields = split("\t",$_);
                ($n,$taxid,$taxon) = ($fields[$nreads_column_index],$fields[$taxid_column_index],$fields[$taxid_column_index+1]);
            }

            if ($taxid)
            {
                if (!exists $nodes{$taxid}) { $unknown_nodes{$taxid} = $taxon; next; }
                $searches{$search_name}->{'n_queries_have_hits'} += $n;
                $searches{$search_name}->{'query_count_by_best_hit_taxid'}->{$taxid} += $n;
            }
            $searches{$search_name}->{'n_queries_total'} += $n;
        }
        close $IN;
        #print $searches{$search_name}->{'n_queries_have_hits'}, " queries have hits in ", scalar(keys %{$searches{$search_name}->{'query_count_by_best_hit_taxid'}}), " taxa<br />\n";

        %{$searches{$search_name}->{'combined_n_queries_per_taxid'}} = ();
        if ($cumulate)
        {
            foreach my $leaf (keys %{$searches{$search_name}->{'query_count_by_best_hit_taxid'}})
            {
                my $n = $searches{$search_name}->{'query_count_by_best_hit_taxid'}->{$leaf};
                for (my $t = $leaf; $t; $t = $nodes{$t}->{'p'}) { $searches{$search_name}->{'combined_n_queries_per_taxid'}->{$t} += $n; }
            }
        }
        else
        {
            foreach my $leaf (keys %{$searches{$search_name}->{'query_count_by_best_hit_taxid'}})
            {
                $searches{$search_name}->{'combined_n_queries_per_taxid'}->{$leaf} = $searches{$search_name}->{'query_count_by_best_hit_taxid'}->{$leaf};
            }
        }
        #print scalar(keys %{$searches{$search_name}->{'combined_n_queries_per_taxid'}}), " taxonomy nodes show up in the tree<br />\n";

        my $krona_file = "$search_name.krona.html";
        if (!-e "$dir/$krona_file") { $krona_file = "krona/$search_name.html"; }
        if (!-e "$dir/$krona_file" and defined($param_value)) { $krona_file = "krona/$param_value/$search_name.html"; }
        if (-e "$dir/$krona_file") { $krona_urls{$search_name} = "$dir_in_url/$krona_file?collapse=false"; }
    }
}

if ($ok) { print_options(); }

if (scalar keys %unknown_nodes) { print scalar(keys %unknown_nodes), " unknown nodes found\n"; }

# Counting total number of queries in each node of the tree.
my %total_combined_n_queries_per_taxid = ();
my $total_n_queries_have_hits = 0;
if ($ok)
{
    foreach my $search_name (keys %searches)
    {
        foreach my $taxid (keys %{$searches{$search_name}->{'combined_n_queries_per_taxid'}})
        {
            $total_combined_n_queries_per_taxid{$taxid} += $searches{$search_name}->{'combined_n_queries_per_taxid'}->{$taxid};
        }
        $total_n_queries_have_hits += $searches{$search_name}->{'n_queries_have_hits'};
    }
    #print "<b>Total:</b><br />\n";
    #print "$total_n_queries_have_hits queries have hits<br />\n";
    #print scalar(keys %total_combined_n_queries_per_taxid), " taxonomy nodes show up in the tree<br />\n";
}

if ($ok and !$show_unranked_nodes) { remove_unranked(1); }

print "</div><div style=\"text-align: center;\"><div class=\"text_tree\">";
my $tree_text = '';
print_node(\$tree_text,1,1,0,$combine_identical_nodes);
$tree_text = add_tree_lines($tree_text);
print $tree_text;
print "</div></div>\n";

print "<div class=\"text\">\n";
print process_template('{HTML footer}');


sub print_options
{
    print "<form action=\"best-hit-tree\" method=\"get\"><div class=\"options\">\n";
    if ($dir ne $default_data_dir) { print "<input type=\"hidden\" name=\"dir\" value=\"", substr($dir,length($default_data_dir)+1), "\">\n"; }
    print "<b>Options</b>\n";
    print "<table cellspacing=\"0\" cellpadding=\"0\" class=\"simple\"><tr><td>\n";
    print "Lowest rank to show: <select name=\"lowest_rank\" size=\"1\">\n";
    foreach my $r (sort { $tax_rank_order{$a} <=> $tax_rank_order{$b} } keys %tax_rank_order)
    {
        if ($r eq 'no rank') { next; }
        print "<option", (($r eq $lowest_rank_to_show) ? " selected=\"selected\"" : ''), ">$r</option>\n";
    }
    print "<option", (($lowest_rank_to_show eq '') ? " selected=\"1\"" : ''), ">show everything</option>\n</select><br />\n";
    print "Only show nodes with at least <input type=\"text\" size=\"6\" maxlength=\"6\" name=\"min_queries\" value=\"$min_n_queries_to_show_a_node\" /> sequences<br />\n";
    print "<input type=\"checkbox\" name=\"show_unranked_nodes\" value=\"1\"" . ($show_unranked_nodes ? ' checked="checked"' : '') . " /> Show unranked nodes<br />\n";
    print "<input type=\"checkbox\" name=\"show_ranks\" value=\"1\"" . ($show_ranks ? ' checked="checked"' : '') . " /> Show taxonomic rank for each node<br />\n";
    print "<input type=\"checkbox\" name=\"combine_identical_nodes\" value=\"1\"" . ($combine_identical_nodes ? ' checked="checked"' : '') . " /> Compress non-branching lineages<br />\n";
    print "<input type=\"checkbox\" name=\"show_abs\" value=\"1\""  . ($show_abs_number ? ' checked="checked"' : '') . " /> Show absolute numbers<br />\n";
    print "<input type=\"checkbox\" name=\"show_perc\" value=\"1\"" . ($show_percentage ? ' checked="checked"' : '') . " /> Show percentage of:<br />\n";
    print "<input type=\"radio\" name=\"prec_rel\" value=\"all\"" . ($show_percentage_out_of_all_queries ? ' checked="checked"' : '') . " /> All queries\n";
    print "&emsp; <input type=\"radio\" name=\"prec_rel\" value=\"have_hits\"" . ($show_percentage_out_of_all_queries ? '' : ' checked="checked"') . " /> Queries that have hits\n";
    print "</td><td style=\"padding-left: 20pt;\">\n";
    print "Searches shown:<br />\n";
    for (my $i=1; $i<=5; $i++)
    {
        my $this_search_name = '';
        foreach my $sn (keys %searches) { if ($searches{$sn}->{'index'} == $i) { $this_search_name = $sn; last; } }
        print "<select name=\"s\" size=\"1\" style=\"color: ", $search_colors[$i], ";\">\n";
        print "<option", (($this_search_name eq '') ? " selected=\"selected\"" : ''), "></option>\n";
        foreach my $sn (sort { versioncmp($a,$b) } keys %all_searches)
        {
            print "<option", (($sn eq $this_search_name) ? " selected=\"selected\"" : ''), ">$sn</option>\n";
        }
        print "</select>\n";
        if (exists $krona_urls{$this_search_name}) { print " <span class=\"link\"><a href=\"" . $krona_urls{$this_search_name} . "\">Visualize</a></span>\n"; }
        if ($i<5) { print "<br />"; }
    }
    if (defined($param_name) and $param_name ne '' and scalar(keys %is_param_value) > 0)
    {
        print "</br></br>";
        print encode_entities($param_name), ': ', "<select name=\"param\" size=\"1\">";
        foreach my $pv (sort { $a <=> $b } keys %is_param_value)
        {
            print "<option", (($pv eq $param_value) ? " selected=\"selected\"" : ''), ">$pv</option>\n";
        }
        print "</select>\n";
    }
    print "</td></tr></table>\n";
    print "<div style=\"text-align: center\"><input type=\"submit\" name=\"submit\" value=\"Update\" style=\"font-weight: bold; margin-top: 10pt;\" /></div>\n";
    print "</div></form>\n";
}

sub print_node
{
    my ($tree_text,$taxid,$show_spaces,$level,$collapse) = @_;
    my $n_queries = $total_combined_n_queries_per_taxid{$taxid};
    if ($show_spaces) { $$tree_text .= (' ')x$level; }
    my $nqt = exists $total_combined_n_queries_per_taxid{$taxid} ? $total_combined_n_queries_per_taxid{$taxid} : 0;
    if ($nodes{$taxid}->{'o'} >= $tax_rank_order{'genus'}) { $$tree_text .= "<i>"; }
    if (!$nqt) { $$tree_text .= '<span style="color:gray">'; }
    $$tree_text .= $nodes{$taxid}->{'n'};
    if (!$nqt) { $$tree_text .= '</span>'; }
    if ($nodes{$taxid}->{'o'} >= $tax_rank_order{'genus'}) { $$tree_text .= "</i>"; }
    if ($show_ranks) { $$tree_text .= ' (' . $nodes{$taxid}->{'r'} . ')'; }
    my @c = ();
    foreach my $cid (@{$nodes{$taxid}->{'c'}})
    {
        my $o = $nodes{$cid}->{'o'};
        my $cnqt = exists $total_combined_n_queries_per_taxid{$cid} ? $total_combined_n_queries_per_taxid{$cid} : 0;
        if ($cnqt >= $min_n_queries_to_show_a_node and
            ($lowest_rank_to_show eq '' or $o <= $tax_rank_order{$lowest_rank_to_show})) { push @c, $cid; }
    }
    if (scalar(@c) == 1 and $total_combined_n_queries_per_taxid{$c[0]} == $n_queries and $collapse)
    {
        #$$tree_text .= " &#x2192; ";
        $$tree_text .= " / ";
        print_node($tree_text,$c[0],0,$level,$collapse);
    }
    else
    {
        if ($show_abs_number or $show_percentage) { $$tree_text .= ' - '; }
        foreach my $sn (sort { $searches{$a}->{'index'} <=> $searches{$b}->{'index'} } keys %searches)
        {
            my $n = 0;
            if (exists $searches{$sn}->{combined_n_queries_per_taxid}->{$taxid}) { $n = $searches{$sn}->{combined_n_queries_per_taxid}->{$taxid}; }
            my $si = $searches{$sn}->{'index'};
            if ($si > 1) { $$tree_text .= '&ensp;'; }
            $$tree_text .= "<span style=\"color: " . $search_colors[$si] . ";\">";
            if ($show_abs_number) { $$tree_text .= commify(sprintf('%.0f',$n)); }
            if ($n > 0 and $show_percentage)
            {
                if ($show_abs_number) { $$tree_text .= ' ('; }
                $$tree_text .= sprintf("%.3f%%",$n*100 /
                               ($show_percentage_out_of_all_queries ? $searches{$sn}->{'n_queries_total'} : $searches{$sn}->{'n_queries_have_hits'}) );
                if ($show_abs_number) { $$tree_text .= ')'; }
            }
            $$tree_text .= "</span>";
        }

        #$$tree_text .= ' - ';
        #$$tree_text .= $n_queries;
        #$$tree_text .= ' sequence' . (($n_queries==1)?'':'s');
        #$$tree_text .= sprintf(" (%.3f%%)",$n_queries*100/$total_n_queries_have_hits);

        #$$tree_text .= "<br />\n";
        $$tree_text .= "\n";

        foreach my $cid (sort { $total_combined_n_queries_per_taxid{$b} <=> $total_combined_n_queries_per_taxid{$a} } @c)
        {
            print_node($tree_text,$cid,1,$level+1,$collapse);
        }
    }
}

sub add_tree_lines
{
    my ($text) = @_;
    my @lines = split /[\r\n]/, $text;
    my $n_lines = scalar(@lines);
    my @line_level = (0) x $n_lines;
    my @line_head = '' x $n_lines;
    for (my $i=0; $i<$n_lines; $i++)
    {
        if ($lines[$i] =~ /^(\s+)(.+)$/) { $line_level[$i] = length($1); $line_head[$i] = ('   ') x $line_level[$i]; $lines[$i] = $2; }
    }
    for (my $i=$n_lines-1; $i>=0; $i--)
    {
        if (!$lines[$i] or !$line_level[$i]) { next; }
        #if (length($line_head[$i]) != $line_level[$i]*3) { die "Line head length is " . length($line_head[$i]) . ", should be " . ($line_level[$i]*3) . "\n"; }
        substr($line_head[$i],($line_level[$i]-1)*3+2,1,'-');
        if (substr($line_head[$i],($line_level[$i]-1)*3+1,1) eq '|') { substr($line_head[$i],($line_level[$i]-1)*3+1,1,'+'); }
        else
        {
            substr($line_head[$i],($line_level[$i]-1)*3+1,1,'L');
            for (my $j=$i-1; $line_level[$j] >= $line_level[$i]; $j--) { substr($line_head[$j],($line_level[$i]-1)*3+1,1,'|'); }
        }
    }
    for (my $i=0; $i<$n_lines; $i++)
    {
        $line_head[$i] =~ s/\-/&#x2500;/g;
        $line_head[$i] =~ s/L/&#x2514;/g;
        $line_head[$i] =~ s/\+/&#x251C;/g;
        $line_head[$i] =~ s/\|/&#x2502;/g;
        #$line_head[$i] =~ s/\s/&ensp;/g;
    }
    my $text_out = '';
    for (my $i=0; $i<$n_lines; $i++)
    {
        $text_out .= '<tt>' . $line_head[$i] . '</tt>';
        $text_out .= ' ' . $lines[$i];
        if ($i < $n_lines-1) { $text_out .= "\n"; }
    }
    return $text_out;
}

sub read_tax_rank_order
{
    my $tax_rank_order_file = 'tax-rank-order.conf';
    my $n = 1;
    open(my $RO,'<',$tax_rank_order_file) or die "Can't open \"$tax_rank_order_file\"\n";
    binmode $RO;
    while (<$RO>) { s/[\x0D\x0A]+$//; $tax_rank_order{$_} = $n++; }
    close $RO;
    #print join('::',sort { $tax_rank_order{$a} <=> $tax_rank_order{$b} } keys %tax_rank_order), "\n";
}

sub compute_tax_order
{
    my ($t) = @_;
    my $r = $nodes{$t}->{'r'};
    $nodes{$t}->{'o'} = ($t == 1) ? 0 : ($r ne 'no rank') ? $tax_rank_order{$r} : $nodes{$nodes{$t}->{'p'}}->{'o'} + 0.01;
    #print $nodes{$t}->{'n'}, ' -- ', sprintf("%.2f",$nodes{$t}->{'o'}), "<br />\n";
    for (@{$nodes{$t}->{'c'}}) { compute_tax_order($_); }
}

sub remove_unranked
{
    my ($t) = @_;
    my @c_list = @{$nodes{$t}->{'c'}};
    if ($t != 1 and $nodes{$t}->{'r'} eq 'no rank')
    {
        my $p = $nodes{$t}->{'p'};
        my @pc = @{$nodes{$p}->{'c'}};
        @{$nodes{$p}->{'c'}} = ();
        foreach my $c (@pc) { if ($c != $t) { push @{$nodes{$p}->{'c'}}, $c; } }
        foreach my $c (@c_list) { push @{$nodes{$p}->{'c'}}, $c; $nodes{$c}->{'p'} = $p; }
    }
    foreach my $cid (@c_list) { remove_unranked($cid); }
}

sub commify { my $text = reverse $_[0]; $text =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g; return scalar reverse $text; }
